@@ -167,8 +167,13 @@ class Base(object):
     def get_interface(self):
         return self.parent.get_interface()
 
+    def get_anchor(self):
+        return "%s:%s" % (
+            self.get_type_name().replace(' ', '-'),
+            self.short_name)
+
     def get_url(self):
-        return "%s#%s" % (self.get_interface().get_url(), self.name)
+        return "%s#%s" % (self.get_interface().get_url(), self.get_anchor())
 
     def _get_generic_with_ver(self, nnode, htmlclass, txt):
         if nnode is None:
@@ -561,6 +566,7 @@ class Property(DBusConstruct, Typed):
 
         immutable = dom.getAttributeNS(XMLNS_TP, 'immutable')
         self.immutable = immutable != ''
+        self.sometimes_immutable = immutable == 'sometimes'
 
         requestable = dom.getAttributeNS(XMLNS_TP, 'requestable')
         self.requestable = requestable != ''
@@ -577,7 +583,9 @@ class Property(DBusConstruct, Typed):
     def get_flag_summary(self):
         descriptions = []
 
-        if self.immutable:
+        if self.sometimes_immutable:
+            descriptions.append("Sometimes immutable")
+        elif self.immutable:
             descriptions.append("Immutable")
 
         if self.sometimes_requestable:
@@ -699,6 +707,9 @@ class Interface(Base):
         self.contact_attributes = build_list(self, ContactAttribute, self.name,
                 dom.getElementsByTagNameNS(XMLNS_TP, 'contact-attribute'))
 
+        self.client_interests = build_list(self, ClientInterest, self.name,
+                dom.getElementsByTagNameNS(XMLNS_TP, 'client-interest'))
+
         # build a list of types in this interface
         self.types = parse_types(self, dom, self.name)
 
@@ -760,6 +771,7 @@ WARNING: Interface not known: '%s'
             (XMLNS_TP, 'handler-capability-token'),
             (XMLNS_TP, 'hct'),
             (XMLNS_TP, 'contact-attribute'),
+            (XMLNS_TP, 'client-interest'),
             (XMLNS_TP, 'simple-type'),
             (XMLNS_TP, 'enum'),
             (XMLNS_TP, 'flags'),
@@ -788,7 +800,7 @@ WARNING: Unknown element(s): %s
 
 class Error(Base):
     def get_url(self):
-        return 'errors.html#%s' % self.name
+        return 'errors.html#%s' % self.get_anchor()
 
     def get_root_namespace(self):
         return self.namespace
@@ -865,7 +877,7 @@ class DBusType(Base):
         else:
             html = 'generic-types.html'
 
-        return '%s#%s' % (html, self.name)
+        return '%s#%s' % (html, self.get_anchor())
 
 class SimpleType(DBusType):
     def get_type_name(self):
@@ -1043,7 +1055,13 @@ class TokenBase(Base):
 
     def __init__(self, parent, namespace, dom):
         super(TokenBase, self).__init__(parent, namespace, dom)
-        self.name = namespace + '/' + self.short_name
+
+        items = [ namespace ]
+
+        if self.short_name != '':
+            items.append (self.short_name)
+
+        self.name = self.separator.join (items)
 
 class ContactAttribute(TokenBase, Typed):
 
@@ -1062,6 +1080,19 @@ class HandlerCapabilityToken(TokenBase):
         assert is_family in ('yes', 'no', '')
         self.is_family = (is_family == 'yes')
 
+class ClientInterest(TokenBase):
+
+    def __init__(self, parent, namespace, dom):
+        super(ClientInterest, self).__init__(parent, namespace, dom)
+
+        self.short_name = self.name
+
+    def get_type_name(self):
+        return 'Client Interest'
+
+    def validate(self):
+        pass
+
 class SectionBase(object):
     """A SectionBase is an abstract base class for any type of node that can
        contain a <tp:section>, which means the top-level Spec object, or any
@@ -1153,18 +1184,12 @@ class Spec(SectionBase):
         for interface in self.interfaces:
                 self.everything[interface.name] = interface
 
-                for method in interface.methods:
-                    self.everything[method.name] = method
-                for signal in interface.signals:
-                    self.everything[signal.name] = signal
-                for property in interface.properties:
-                    self.everything[property.name] = property
-                for property in interface.tpproperties:
-                    self.everything[property.name] = property
-                for token in interface.contact_attributes:
-                    self.everything[token.name] = token
-                for token in interface.handler_capability_tokens:
-                    self.everything[token.name] = token
+                for things in [ 'methods', 'signals', 'properties',
+                                'tpproperties', 'contact_attributes',
+                                'handler_capability_tokens',
+                                'client_interests' ]:
+                    for thing in getattr(interface, things):
+                        self.everything[thing.name] = thing
 
                 for type in interface.types:
                     self.types[type.name] = type
